# ------------------------------------------------------------------------------
# Licence:
# Copyright (c) 2012-2018 Luzzi Valerio
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.
#
#
# Name:        abstractdb.py
# Purpose:
#
# Author:      Luzzi Valerio
#
# Created:     31/07/2018
# ------------------------------------------------------------------------------
import os,sys,re
from random import randint
from .sqlitedb import *
from .sqlite_utils import *
from opensitua_core import system_mail


class UsersDB(SqliteDB):
    """
    UsersDB - a class with common base methods
    """
    def __init__(self, dsn=":memory:", modules="", fileconf="", filekey="", verbose=False):
        """
        Constructor
        """
        SqliteDB.__init__(self, dsn)
        self.create_function("md5", 1, sqlite_md5 )
        self.__create_structure__(verbose)
        self.fileconf = fileconf
        self.filekey  = filekey

    def __create_structure__(self, verbose=False):
        """
        __create_structure__
        """
        sql = """
        CREATE TABLE IF NOT EXISTS [users](
            mail TEXT PRIMARY KEY,
            name TEXT, 
            token TEXT(32), 
            enabled BOOL DEFAULT 0,
            role TEXT DEFAULT 'user'
        );"""
        self.execute(sql)

    def getAdmins(self, verbose=False):
        administrators = self.execute("""SELECT [mail] FROM [users] WHERE [role] ='admin';""", None, outputmode="array", verbose=verbose)
        administrators = [ item for (item,) in administrators]
        return administrators

    def sendMail(self, To, Object, text, env=None, verbose=False):
        system_mail(To, text, Object, "", self.fileconf, self.filekey, env)

    def sendMailToAdmin(self, Object, text , env=None, verbose=False):
        administrators = self.getAdmins()
        system_mail(administrators, text, Object, "", self.fileconf, self.filekey, env)

    def sendMailAndFeedBackAdmin(self, To, ObjectForUser, ObjectForAdmin="", text="", env=None, verbose=False):
        ObjectForAdmin = ObjectForAdmin if ObjectForAdmin else ObjectForUser
        system_mail(To, text, ObjectForUser, "", self.fileconf, self.filekey, env)
        system_mail(self.getAdmins(), text, ObjectForAdmin, To, self.fileconf, self.filekey, env)

    def addUser(self, mail, name="", password="", role="user", enabled=False, sendmail=False, url="localhost", extra="", verbose=False):
        """
        addUser
        """
        password = sqlite_md5(password) if password else sqlite_md5("%s"%(randint(0,100000)))[:5]
        env = {
            "mail":mail,
            "name": name if name else mail,
            "password": password,
            "role":role,
            "enabled":1 if enabled else 0,
            "url":url
        }
        sql= """
        INSERT OR IGNORE INTO [users]([mail],[name],[token],[role],[enabled]) VALUES('{mail}','{name}',md5('{mail}'||'{password}'),'{role}',{enabled});
        SELECT [token] FROM   [users] WHERE [name] ='{name}' AND [mail]='{mail}';
        """
        __token__ = self.execute(sql, env, outputmode="scalar", verbose=verbose)
        env["__token__"] = __token__

        if sendmail:
            text = ""  # some headers
            text+= "%s"%(extra)
            self.sendMailToAdmin( """Check user request of {name}""" , text, env )

        return __token__

    def existsUser(self, name, mail=""):
        """
        name
        """
        sql ="""SELECT COUNT(*) FROM [users] WHERE [mail] = '{mail}';"""
        count = self.execute(sql, {"name":name,"mail":mail}, outputmode="scalar")
        return count>0


    def enableUser(self, token, enabled=1, sendmail=False, url="localhost", subject="", verbose=False):
        """
        enableUser
        """
        env ={
            "token":token,
            "enabled":1 if enabled else 0,
            "password":  sqlite_md5("%s"%randint(0,10000))[:5]
        }
        #Abilita l'utente e gli cambia il token

        sql = """
        UPDATE [users] SET [enabled]={enabled},[token]=md5([mail]||'{password}') WHERE [token]='{token}';
        SELECT [mail],[name],[enabled],md5([mail]||'{password}') as [token] FROM [users]  WHERE [token]=md5([mail]||'{password}');
        """
        (mail,name,enabled,token) = self.execute(sql, env, outputmode='first-row', verbose=verbose)

        # A mail to the user
        if mail:
            user = {
                "name":name,
                "mail":mail,
                "enabled":enabled,
                "password":env["password"],
                "token":token,
                "url":url
            }
            #Login at <a href='http://{url}/webgis/private/{mail}'>http://localhost/webgis/</a></br>
            text = """</br>
                    Hello {name}!</br>
                    Login at <a href='{url}'>{url}</a></br>
                    Your password is:<b>{password}</b></br>
                    """

            #if sendmail and os.path.isfile(self.fileconf):
            #    subject = subject if subject else "Credentials for the Web-Tool."
            #    system_mail(mail, text, subject, self.fileconf, self.filekey, user, verbose=verbose)
            ObjectForUser = subject if subject else "Credentials for the Web-Tool."
            ObjectForAdmin = "New User {name} has been enabled!"
            self.sendMailAndFeedBackAdmin(mail, ObjectForUser,ObjectForAdmin, text, user, verbose=verbose)
            return user

        return False

    def getToken(self,username, password):
        """
        getToken
        """
        env = {
            "username": username,
            "password": password
        }
        sql = """
        SELECT md5([token]||strftime('%Y-%m-%d','now')) FROM [users]
            WHERE [mail] = '{username}'
            AND [token] LIKE md5([mail]||'{password}')
            AND [enabled];
        """
        return self.execute(sql, env, outputmode="scalar", verbose=False)

    def isValid(self, username, token):
        """
        checkToken -  check token is valid
        """
        env = {
            "__username__": username,
            "__token__": token
        }
        sql = """
        SELECT md5([token]||strftime('%Y-%m-%d','now'))='{__token__}' FROM [users] WHERE [mail] = '{__username__}' LIMIT 1;
        """
        return self.execute(sql, env, outputmode="scalar", verbose=False)

    def changePassword(self, mail, sendmail=False, service="localhost"):
        """
        changePassword
        """
        password = sqlite_md5("%s" % (randint(0, 100000)))[:5]
        env = {
            "mail": mail,
            "password": password,
            "service": service
        }
        sql = """
              UPDATE [users] SET [token]=md5([mail]||'{password}') WHERE [mail] = '{mail}';
              SELECT [name] FROM [users] WHERE [mail] = '{mail}';
        """
        env["name"] = self.execute(sql, env, outputmode="scalar")

        if sendmail and os.path.isfile(self.fileconf):

            Subject = """Password change for {name} in {service}"""

            text = """
            <b>{name}</b> ({mail})<br>
            Your password has been changed:<br>
            password:<b>{password}</b>
            """
            #system_mail(mail, text, Subject, self.fileconf, self.filekey, env)
            ObjectForUser = Subject
            ObjectForAdmin = "User {name} changed the password"
            self.sendMailAndFeedBackAdmin(mail, ObjectForUser, ObjectForAdmin, text, env)

        return password


# ------------------------------------------------------------------------------
# Licence:
# Copyright (c) 2012-2021 Luzzi Valerio
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.
#
#
# Name:        jobsdb.py
# Purpose:
#
# Author:      Luzzi Valerio
#
# Created:     12/03/2021
# ------------------------------------------------------------------------------
import os,sys,re
import subprocess, psutil
from .sqlitedb import *
from .sqlite_utils import *


class JobsDB(SqliteDB):
    """
    JobsDB - a class with common base methods
    """
    def __init__(self, dsn=":memory:", modules="", fileconf="", filekey="", verbose=False):
        """
        Constructor
        """
        SqliteDB.__init__(self, dsn)
        self.create_function("md5", 1, sqlite_md5 )
        self.__create_structure__(verbose)

    def __create_structure__(self, verbose=False):
        """
        __create_structure__
        """
        sql = """
        CREATE TABLE IF NOT EXISTS [jobs](
              [jid] INTEGER, 
              [pid] INT, 
              [type] TEXT, 
              [case_study] TEXT, 
              [user] TEXT, 
              [description] TEXT, 
              [command] TEXT, 
              [status] TEXT DEFAULT ready, 
              [progress] INT DEFAULT 0, 
              [inserttime] INTEGER DEFAULT (STRFTIME ('%s', 'now', 'localtime')), 
              [starttime] DATETIME, 
              [endtime] DATETIME, 
              PRIMARY KEY([jid], [user], [case_study])) WITHOUT ROWID;
        );"""
        self.execute(sql)


    def addJob(self, options, verbose=False):
        """
        addUser
        """

        env = {
            "jid":"0001",
            "pid":os.getpid(),
            "type":"generic",
            "user":"unknown",
            "description":"...",
            "command":"",
            "status":"ready",
            "progress":0.0
        }
        env.update(options)
        sql= """
        INSERT OR IGNORE INTO [jobs]([jid],[pid],[type],[user],[description],[command],[status],[progress]) 
                            VALUES( '{jid}','{pid}','{type}','{user}','{description}','{command}','{status}','{progress}');
        """
        self.execute(sql, env)

    def executeJob(self,jid):
        """
        executeJob
        """
        params["jid"] = jid
        sql = """UPDATE [jobs] SET status='running',progress=0 WHERE jid='{jid}';
                     SELECT [command] FROM [jobs] WHERE [jid]='{jid}' and status='running';"""
        command = self.execute(sql, params, outputmode="scalar")

        if command:
            try:
                p = subprocess.Popen(command)
                print("Running <%s> with pid:%d" % (command, p.pid))
                print("-----------------------------------------")
                params["pid"] = p.pid
                params["starttime"] = strftime('%Y-%m-%d %H:%M:%S', None)
                res = {"success": True, "data": "process started"}
            except Exception as ex:
                params["pid"] = -1
                params["status"] = "error"
                params["starttime"] = ""
                res = {"success": False, "exception": "unable to start process"}
        else:
            params["pid"] = -1
            params["status"] = "error"
            params["starttime"] = ""
            res = {"success": False, "exception": "unable to find process with jobid={jid}".format(**params)}
            print(params["status"])

        sql = """UPDATE [jobs] SET status='{status}', pid='{pid}', progress=0, starttime='{starttime}' WHERE jid='{jid}';"""
        self.execute(sql, params)

    def ProcessQueue(self, parallelism = -1, max_load = 70, verbose = False):
        """
        ProcessQueue - process the job queue
        """
        parallelism = parallelism if parallelism else psutil.cpu_count()
        sql = """SELECT * FROM [jobs] WHERE [status] = 'queued' ORDER BY [inserttime] ASC;"""
        job_list = self.execute(sql, outputmode="dict", verbose=verbose)
        for job in job_list:
            n = self.execute(
                """SELECT COUNT(*) FROM [jobs] WHERE [status] NOT IN ('ready','queued','done','error');""",
                outputmode="scalar", verbose=verbose)
            cpu_load = psutil.cpu_percent(interval=1)
            if n < parallelism or cpu_load < max_load:
                self.ExecuteJob(job["jid"])







# ------------------------------------------------------------------------------
# Licence:
# Copyright (c) 2012-2020 Luzzi Valerio
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.
#
#
# Name:        abstractdb.py
# Purpose:
#
# Author:      Luzzi Valerio
#
# Created:     31/07/2018
# ------------------------------------------------------------------------------
import os,sys,re
import time
import psycopg2
from opensitua_core import *

SQL_INJECTION_WORDS = (

    "CREATE ",
    "INSERT ",
    "UPDATE ",
    "REPLACE ",
    "SELECT "
    "DROP ",
    "DELETE ",

)


class AbstractDB:
    """
    AbstractDB - a class with common base methods
    """
    def close(self, verbose=False):
        """
        Close the db connection
        """
        if self.conn:
            if verbose:
                print("closing db...")
            self.conn.close()

    def commit(self):
        """
        force commit
        """
        if self.conn:
            self.conn.commit()

    def __del__(self):
        """
        destructor
        """
        self.close()

    def __get_cursor__(self):
        """
        __get_cursor__
        """
        return self.conn.cursor()

    def __connect__(self):
        pass

    def __contains__(self, text, items, case_sensitive=True):
        """
        __contains__
        """
        for item in items:
            if case_sensitive and "%s"%item in text:
                return True
            elif not case_sensitive and ("%s"%item).upper() in text.upper():
                return True
        return False

    def __sql_injection__(self, text):
        """
        avoid sql injection
        """
        if isinstance(text,str) and self.__contains__(text, SQL_INJECTION_WORDS, False):
            print("Warning:SQL INJECTION DETECTED!:<%s>"%text)
            return ""
        #if text.strip('\t\n\r ').startswith("'")

        return text

    def __check_args__(self, env):
        """
        check for sql injection
        """
        for key in env:
            env[key] = self.__sql_injection__(env[key])
        return env

    def __prepare_query__(self, sql, env={}, verbose=False):
        """
        prepare the query
        remove comments and blank lines
        """
        comment1 = "--"
        comment2 = "#"
        comment3 = "//"
        sql = re.sub(r'(\r\n|\n)','\n',sql)
        lines = split(sql, "\n", "'\"")

        # follow statement remove comments after SQL line code.
        lines = [split(line, comment1, "'\"")[0] for line in lines]
        lines = [split(line, comment2, "'\"")[0] for line in lines]
        lines = [split(line, comment3, "'\"")[0] for line in lines]
        lines = [line.strip(" \t") for line in lines]

        # follow statement remove all lines of comments
        lines = [line for line in lines if len(line) > 0 and not line.startswith(comment1)]
        lines = [line for line in lines if len(line) > 0 and not line.startswith(comment2)]
        lines = [line for line in lines if len(line) > 0 and not line.startswith(comment3)]

        sql = " ".join(lines)
        #remove spaces between stetements
        sql = re.sub(r';\s+',';',sql)

        #check for sql injection
        env = self.__check_args__(env)

        return sql, env

    def execute(self, sql, env=None, outputmode="array", commit=True, verbose=False):
        """
        Make a query statement list
        Returns a cursor
        """
        rows = []
        cursor = self.__get_cursor__()
        env = env.copy() if env else {}
        env.update(os.environ)
        if cursor:
            sql, env = self.__prepare_query__(sql, env, verbose)

            sql = sformat(sql, env)
            commands = split(sql, ";", "'\"")
            commands = [command.strip() + ";" for command in commands if len(command) > 0]

            for command in commands:
                t1 = time.time()
                command = sformat(command, env)

                try:

                    cursor.execute(command)

                    if commit==True and not command.upper().strip(' \r\n').startswith("SELECT"):
                         self.conn.commit()

                    env.update(os.environ)

                    t2 = time.time()

                    if verbose:
                        command = command.replace("\n", " ")
                        print("->%s:Done in (%.4f)s" % (command[:], (t2 - t1)))

                except Exception as ex:
                    command = command.replace("\n", " ")
                    print( "No!:SQL Exception:%s :(%s)"%(command,ex))

                    if outputmode in ("response","row-response"):
                        res = {"status": "fail", "success": False, "exception": "%s"%ex, "sql": command}
                        return res

            if outputmode == "cursor":
                return cursor

            elif outputmode == "array":
                for row in cursor:
                    rows.append(row)

            elif outputmode == "scalar":
                row = cursor.fetchone()
                if row and len(row):
                    return row[0]
                return None

            elif outputmode == "first-row":
                row = cursor.fetchone()
                if row and len(row):
                    return row
                if cursor.description:
                    return tuple([None]*len(cursor.description))
                return None

            elif outputmode == "table":
                metadata = cursor.description
                if metadata:
                    rows.append(tuple([item[0] for item in metadata]))
                for row in cursor:
                    rows.append(row)

            elif outputmode in ( "object", "dict" ):
                if cursor.description:
                    columns = [item[0] for item in cursor.description]
                    for row in cursor:
                        line = {}
                        for j in range(len(row)):
                            line[columns[j]] = row[j]
                        rows.append(line)

            elif outputmode == "columns":
                n = len(cursor.description)
                rows = [[] for j in range(n)]
                for row in cursor:
                    for j in range(n):
                        rows[j].append(row[j])

            elif outputmode in ("response","row-response"):
                metadata = []
                res = {}
                if cursor.description:
                    metadata = cursor.description
                    columns = [item[0] for item in cursor.description]
                    for row in cursor:
                        line = {}
                        for j in range(len(row)):
                            line[columns[j]] = row[j]
                        rows.append(line)

                    res = {"status": "success", "success": True, "data": rows, "metadata": columns, "pos":0, "total_count":len(rows), "exception": None}
                return res

            elif outputmode == "column-response":
                metadata = []
                res = {}
                if cursor.description:
                    metadata = cursor.description
                    columns = [item[0] for item in cursor.description]
                    selection={}
                    for row in cursor:
                        for j in range(len(row)):
                            if not columns[j] in selection:
                                selection[columns[j]] = []
                            selection[columns[j]].append(row[j])

                    res = {"status": "success", "success": True, "data": selection, "metadata": metadata, "exception": None}
                return res

        return rows

    def executeMany(self, sql, env={}, values=[], commit=True, verbose=False):
        """
        Make a query statetment
        """
        cursor = self.__get_cursor__()
        line = sformat(sql, env)
        try:
            t1 = time.time()
            cursor.executemany(line, values)
            if commit:
                self.conn.commit()
            t2 = time.time()
            if verbose:
                line = line.replace("\n", " ")
                print("->%s:Done in (%.2f)s" % (line[:], (t2 - t1)))

        except Exception as ex:
            line = line.replace("\n", " ")
            print( "No!:SQL Exception:%s :(%s)"%(line,ex))

    def insertMany(self, tablename, values=[], mode ="IGNORE", fieldnames = "",commit=True, verbose=False):
        """
        Make an insert statetment
        """
        cursor = self.__get_cursor__()
        fieldnames = wrap(listify(fieldnames), "[", "]")
        if len(values):
            n = len(fieldnames) if len(fieldnames) else len(values)

            env ={
                "mode"        :   mode,
                "tablename"   : tablename,
                "fieldnames"  : ",".join( fieldnames ),
                "question_marks" : ",".join( ["?"]*n )
            }
            sql = """INSERT OR {mode} INTO [{tablename}]({fieldnames}) VALUES ({question_marks});"""
            self.executeMany(sql, env, values, commit, verbose)

    def list(self, verbose=False):
        """
        list tables
        """
        sql = """
        SELECT [tbl_name] FROM [sqlite_master] WHERE LOWER([type]) in ('table','view')
        UNION ALL
        SELECT [tbl_name] FROM [sqlite_temp_master] WHERE LOWER([type]) in ('table','view');
        """
        return self.execute(sql, {}, outputmode="array", verbose=verbose)





#-------------------------------------------------------------------------------
# Licence:
# Copyright (c) 2012-2020 Valerio for Gecosistema S.r.l.
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.
#
#
# Name:        opensitua_database
# Purpose:
#
# Author:      Luzzi Valerio
#
# Created:    23/07/2020
#-------------------------------------------------------------------------------
import os,sys,re
import json
from .postgresdb import *

def sformat(text, args):
    """
    sformat
    """
    for key in args:
        text = text.replace("{%s}" % key, "%s" % (args[key]))

    text = re.sub(r'\{(.*?)\}',r'',text)
    return text


def JSONResponse(obj, start_response=None):
    """
    JSONResponse
    """
    if isinstance(obj, str):
        text = obj
    elif isinstance(obj, (dict, list)):
        text = json.dumps(obj)
    else:
        text = obj

    response_headers = [('Content-type', 'application/json'), ('Content-Length', str(len(text)))]
    if start_response:
        start_response("200 OK", response_headers)
    return [text.encode('utf-8')]

def SqlScriptResponse( params, start_response ):
    """
    SqlScriptResponse - exec an sctipt
    """
    res = []
    db = None
    sql = ""
    verbose = False

    #params = Params(environ).toDictionary()
    DOCUMENT_WWW = params["DOCUMENT_WWW"]
    filename = params["filename"] if "filename" in params else ""
    filename = DOCUMENT_WWW + "/" + filename
    filename = re.sub(r'\.\.', '', filename)

    if os.path.isfile(filename):
        with open(filename, 'r') as f:
            line = f.readline()
            while line:
                # search for debug mode
                pattern = r'^\#\s*(DEBUG\s*=\s*TRUE).*'
                m = re.search(pattern, line, re.I | re.S)
                verbose = True if m else False

                pattern = r'^\s*SELECT\s+[\',\"](?P<dsn>.*host\s*=.*)[\',\"]\s*;'
                m = re.search(pattern, line, re.I | re.S)
                dsn = m.group('dsn') if m else ""
                if dsn:
                    if db:
                        db.close()
                    db = PostgresDB(dsn)
                    sql = ""
                else:
                    # catenate text
                    sql += line

                line = f.readline()

            if db and sql:
                # print(sformat(sql,params))
                res = db.execute(sql, params, outputmode="dict", verbose=verbose)

            if db:
                db.close()

            return JSONResponse(res, start_response)

    return JSONResponse({"exception": "filename <%s> doesnot exits!" % (filename)}, start_response)







# ------------------------------------------------------------------------------
# Licence:
# Copyright (c) 2012-2018 Luzzi Valerio
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.
#
#
# Name:        abstractdb.py
# Purpose:
#
# Author:      Luzzi Valerio
#
# Created:     31/07/2018
# ------------------------------------------------------------------------------
import os,sys,re
import psycopg2
from .abstractdb import *
from opensitua_core import *

class PostgresDB(AbstractDB):
    """
    PostgresDB - a class with common base methods
    """
    def __init__(self, dsn="", env={}, verbose=False):
        """
        Constructor
        """
        self.dsn = sformat(dsn,env)
        self.conn = None
        self.__connect__()

    def __connect__(self):
        """
        __connect__
        """
        try:
            self.conn = psycopg2.connect(self.dsn)

        except Exception as err:
            print(err)
            self.close()




# ------------------------------------------------------------------------------
# Licence:
# Copyright (c) 2012-2020 Luzzi Valerio
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.
#
#
# Name:        settingsdb.py
# Purpose:
#
# Author:      Luzzi Valerio
#
# Created:     18/09/2019
# ------------------------------------------------------------------------------
import os,sys,re
from .sqlitedb import *
from .sqlite_utils import *


class SettingsDB(SqliteDB):
    """
    SettingsDB - a class with common base methods
    """
    def __init__(self, dsn=":memory:", modules="", verbose=False):
        """
        Constructor
        """
        SqliteDB.__init__(self, dsn)
        #self.create_function("md5", 1, sqlite_md5 )
        self.__create_structure__(verbose)

    def __create_structure__(self, verbose=False):
        """
        __create_structure__
        """
        sql = """
        CREATE TABLE IF NOT EXISTS [settings](
            key    TEXT,
            value  TEXT DEFAULT NULL, 
            [type] TEXT DEFAULT 'string',
            [groupname] TEXT DEFAULT 'General',
            PRIMARY KEY([key],[groupname])
        );"""
        self.execute(sql)

    def set(self, key, value, type='string', groupname='General'):
        """
        set - add or update a key,value tuple
        """
        sql ="""INSERT OR REPLACE INTO [settings]([key],[value],[type],[groupname]) VALUES(?,?,?,?);"""
        self.executeMany(sql,{},[(key,value,type,groupname)])

    def unset(self, key, groupname='General'):
        """
        unset - remove key,value tuple
        """
        sql = """DELETE FROM [settings] WHERE [key]=? AND [groupname]=?;"""
        self.executeMany(sql, {}, [(key, groupname)])

    def get(self, key, groupname='General'):
        """
        get - get a key, value
        """
        (value,type) = self.execute("""SELECT [value],[type] FROM [settings] WHERE [key] LIKE '{key}' AND [groupname]='{groupname}' LIMIT 1;""",{"key":key,"groupname":groupname},outputmode="first-row")
        type = lower(type)
        if type == "string":
            return "%s"%value
        elif type=="number":
            return float(value)
        elif type=="list":
            return value.split(",")
        else:
            return value


#-------------------------------------------------------------------------------
# Licence:
# Copyright (c) 2012-2019 Valerio for Gecosistema S.r.l.
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.
#
#
# Name:        opensitua_database
# Purpose:
#
# Author:      Luzzi Valerio
#
# Created:    08/07/2019
#-------------------------------------------------------------------------------
import pkg_resources

def get_version():
    return pkg_resources.get_distribution(PACKAGE_NAME)


from .sqlitedb import *
from .postgresdb import *
from .usersdb import *
from .settingsdb import *
from .http_utils import *
from .jobsdb import *



# ------------------------------------------------------------------------------
# Licence:
# Copyright (c) 2012-2018 Luzzi Valerio
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.
#
#
# Name:        abstractdb.py
# Purpose:
#
# Author:      Luzzi Valerio
#
# Created:     31/07/2018
# ------------------------------------------------------------------------------
import os,sys,re
import time
import sqlite3 as sqlite
from .abstractdb import *
from opensitua_core import *

class SqliteDB(AbstractDB):
    """
    SqliteDB - a class with common base methods
    """
    def __init__(self, dsn=":memory:", modules="", verbose=False):
        """
        Constructor
        """
        mkdirs(justpath(dsn))
        self.dsn = dsn
        self.conn = None
        self.__connect__()
        self.pragma("synchronous=OFF", verbose=verbose)
        self.pragma("journal_mode=WAL", verbose=verbose)
        self.pragma("foreign_keys=ON", verbose=verbose)
        self.pragma("cache_size=4096", verbose=verbose)
        self.load_extension(modules, verbose=verbose)
        self.conn.enable_load_extension(True)

    def __connect__(self):
        """
        __connect__
        """
        try:
            if not self.dsn.startswith(":"):
                self.dsn = forceext(self.dsn, "sqlite") if justext(self.dsn) == "" else self.dsn
            self.conn = sqlite.connect(self.dsn)

        except sqlite.Error as err:
            print(err)
            self.close()

    def pragma(self, text, env={}, verbose=True):
        """
        pragma
        """
        try:
            text = sformat(text,env)
            if verbose:
                print("PRAGMA " + text)
            self.conn.execute("PRAGMA " + text)
        except sqlite.Error as err:
            print(err)

    def create_function(self, func, nargs, fname):
        """
        create_function
        """
        self.conn.create_function(func, nargs, fname)

    def create_aggregate(self, func, nargs, fname):
        """
        create_aggregate
        """
        self.conn.create_aggregate(func, nargs, fname)

    def load_extension(self, modules, verbose=False):
        """
        load_extension
        """
        try:
            modules = listify(modules)
            self.conn.enable_load_extension(True)
            if isLinux() or isMac():
                modules = [os.join(justpath(item), juststem(item)) for item in modules]

            for module in modules:
                try:
                    self.conn.execute("SELECT load_extension('%s');" % (module))
                    if verbose:
                        print("loading module %s...ok!" % (module))
                except OperationalError as ex:
                    print("I can't load  %s because:%s" % (module,ex))

            self.conn.enable_load_extension(False)
        except(Exception):
            print("Unable to load_extension...")




import os,re
import setuptools

VERSION         =  "0.0.89"

PACKAGE_NAME    = "opensitua_database"
AUTHOR          = "Valerio Luzzi"
EMAIL           = "valluzzi@gmail.com"
GITHUB          = "https://github.com/valluzzi/%s.git"%(PACKAGE_NAME)
DESCRIPTION     = "A core database functions package"


setuptools.setup(
    name=PACKAGE_NAME,
    version=VERSION,
    author=AUTHOR,
    author_email=EMAIL,
    description=DESCRIPTION,
    long_description=DESCRIPTION,
    url=GITHUB,
    packages=setuptools.find_packages(),
    classifiers=(
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ),
    install_requires=['opensitua_core','psycopg2']
)
